/*
 * #############################################################################
 * ATTENTION: BridgeMain must be started because of BridgeRMI (design fault) !?!
 * #############################################################################
 * 
 * @author Walther Neuper, member of the ISAC-team, 
 * Copyright (c) 2004 by Walther Neuper 
 * created Aug 30, 2004, 11:58:35 AM
 * Institute for Softwaretechnology, Graz University of Technology, Austria.
 * 
 * Use is subject to license terms.
 */
package isac.bridge;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.Vector;

import org.apache.log4j.Logger;

import isac.interfaces.ICalcElement;
import isac.interfaces.ICalcIterator;
import isac.interfaces.IToCalc;
import isac.interfaces.IToUser;
import isac.kestore.KEStoreServices;
import isac.util.CalcChanged;
import isac.util.Formalization;
import isac.util.formulae.CalcHead;
import isac.util.formulae.CalcFormula;
import isac.util.formulae.Position;
import isac.util.parser.FormalizationDigest;
import isac.util.tactics.Tactic;
import junit.framework.TestCase;

/**
 * @author Walther Neuper Aug 30, 2004, 11:58:35 AM
 */
public class TestBridgeInput extends TestCase {

    static Logger logger_ = Logger.getLogger(MathEngine.class.getName());

    /**
     * uses the example 'IsacCore/Tests/minipbl_with_mini-subpbl' comprises the
     * usecase \label{SOLVE:MANUAL:FORMULA:enter} see
     * smltest/FE-interface/interface.sml "...enter} right"
     */
    public void testAppendRight() throws Exception {
        System.out
                .println("---BEGIN isac.bridge.TestBridgeInput#testAppendRight");

        //load the file of the example,
        //see TestKEStoreServices#testLoadContentMinisubpbl
        KEStoreServices ks = new KEStoreServices("isac/xmldata");
        Vector vec = ks.loadContent("exp", "exp_IsacCore_Tests_1b.xml");
        //parse out the Formalization
        //see TestFormalizationDigest#testParseExplMini
        String xmlstr = (String) vec.firstElement();
        FormalizationDigest fd = new FormalizationDigest();
        Formalization fmz = fd.parseFormalization((String) xmlstr);
        System.out.println(".end step_a-----");

        //start the calculation
        MathEngine.init("localhost");
        MathEngine me = MathEngine.getMathEngine();

        IToCalc ct = me.getCalcTree(fmz);
        IToUser user = new MockIToUser();
        ct.addDataChangeListener(user);
        CalcChanged cce = null;
        System.out.println(".end step_a-----");
        //end step_a
        // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        //do the first two steps
        ct.completeCalcHead();
        ct.autoCalculate(IToCalc.SCOPE_CALCULATION, 1);// x + 1 = 2
        ct.autoCalculate(IToCalc.SCOPE_CALCULATION, 1);// x + 1 + -1 * 2 = 0
        //		ICalcIterator ci = ct.getActiveFormula();
        //		Formula fa = (Formula) ci.getFormula();
        //		assertEquals("x + 1 + -1 * 2 = 0",fa.toSMLString());
        //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        //append "-1 + x = 0" which the kernel would have proposed, too
        CalcFormula in = new CalcFormula();
        in.setText("-1 + x = 0");
        ct.appendFormula(in);//<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
        cce = ((MockIToUser) user).getCalcChangedEvent();
        assertEquals("unchanged", cce.getLastUnchangedFormula().toSMLString(),
                "([1],Res)");
        assertEquals("deleted", cce.getLastDeletedFormula().toSMLString(),
                "([1],Res)");
        assertEquals("generated", cce.getLastGeneratedFormula().toSMLString(),
                "([2],Res)");

        //no formulae have been inserted:
        Vector elems = cce.getLastUnchangedFormula().getFormulaeFromTo(
                cce.getLastGeneratedFormula(), new Integer(2), false);
        assertEquals("the input formula is returned:", ((CalcFormula) elems
                .firstElement()).toSMLString(), "-1 + x = 0");
        assertEquals("only _one_ formula returned", elems.size(), 1);

        cce.getLastUnchangedFormula().moveDown();
        assertEquals(cce.getLastUnchangedFormula().toSMLString(), cce
                .getLastGeneratedFormula().toSMLString());
        logger_.debug("---END isac.bridge.TestBridgeInput#testAppendRight");
    }

    /**
     * uses the example 'IsacCore/Tests/minipbl_with_mini-subpbl' comprises the
     * usecase \label{SOLVE:MANUAL:FORMULA:enter} see
     * smltest/FE-interface/interface.sml "...enter} other"
     */
    public void testAppendOther1() throws Exception {
        System.out
                .println("---BEGIN isac.bridge.TestBridgeInput#testAppendOther1");

        //load the file of the example,
        //see TestKEStoreServices#testLoadContentMinisubpbl
        KEStoreServices ks = new KEStoreServices("isac/xmldata");
        Vector vec = ks.loadContent("exp", "exp_IsacCore_Tests_1b.xml");
        //parse out the Formalization
        //see TestFormalizationDigest#testParseExplMini
        String xmlstr = (String) vec.firstElement();
        FormalizationDigest fd = new FormalizationDigest();
        Formalization fmz = fd.parseFormalization((String) xmlstr);
        System.out.println(".end step_a-----");

        //start the calculation
        MathEngine.init("localhost");
        MathEngine me = MathEngine.getMathEngine();

        IToCalc ct = me.getCalcTree(fmz);
        IToUser user = new MockIToUser();
        ct.addDataChangeListener(user);
        CalcChanged cce = null;
        System.out.println(".end step_a-----");
        //end step_a
        // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        //do the first two steps
        ct.completeCalcHead();
        ct.autoCalculate(IToCalc.SCOPE_CALCULATION, 1);// x + 1 = 2
        ct.autoCalculate(IToCalc.SCOPE_CALCULATION, 1);// x + 1 + -1 * 2 = 0
        //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        //append "x - 1 = 0" which needs 11 steps (on same level) to justify
        CalcFormula in = new CalcFormula();
        in.setText("x - 1 = 0");
        ct.appendFormula(in);//<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
        cce = ((MockIToUser) user).getCalcChangedEvent();
        assertEquals("unchanged", cce.getLastUnchangedFormula().toSMLString(),
                "([1],Res)");
        assertEquals("deleted", cce.getLastDeletedFormula().toSMLString(),
                "([1],Res)");
        assertEquals("generated", cce.getLastGeneratedFormula().toSMLString(),
                "([2],Res)");

        //the inserted formulae need _NOT_ to be displayed:
        Vector elems = cce.getLastUnchangedFormula().getFormulaeFromTo(
                cce.getLastGeneratedFormula(), new Integer(0), //not displayed
                false);
        assertEquals("the input formula is returned:", ((CalcFormula) elems
                .firstElement()).toSMLString(), "x - 1 = 0");
        assertEquals("only _one_ formula returned", elems.size(), 1);

        //but there are 11 steps in between
        elems = cce.getLastUnchangedFormula().getFormulaeFromTo(
                cce.getLastGeneratedFormula(), new Integer(2), // displayed
                false);
        assertEquals("elems could be displayed ", elems.size(), 11);
        logger_.debug("---END isac.bridge.TestBridgeInput#testAppendOther1");
    }

    /**
     * uses the example 'IsacCore/Tests/minipbl_with_mini-subpbl' comprises the
     * usecase \label{SOLVE:MANUAL:FORMULA:enter} see
     * smltest/FE-interface/interface.sml "...enter} oth 2"
     */
    public void testAppendOther2() throws Exception {
        System.out
                .println("---BEGIN isac.bridge.TestBridgeInput#testAppendOther2");

        //load the file of the example,
        //see TestKEStoreServices#testLoadContentMinisubpbl
        KEStoreServices ks = new KEStoreServices("isac/xmldata");
        Vector vec = ks.loadContent("exp", "exp_IsacCore_Tests_1b.xml");
        //parse out the Formalization
        //see TestFormalizationDigest#testParseExplMini
        String xmlstr = (String) vec.firstElement();
        FormalizationDigest fd = new FormalizationDigest();
        Formalization fmz = fd.parseFormalization((String) xmlstr);
        System.out.println(".end step_a-----");

        //start the calculation
        MathEngine.init("localhost");
        MathEngine me = MathEngine.getMathEngine();

        IToCalc ct = me.getCalcTree(fmz);
        IToUser user = new MockIToUser();
        ct.addDataChangeListener(user);
        CalcChanged cce = null;
        System.out.println(".end step_a-----");
        //end step_a
        // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        //do the first two steps
        ct.completeCalcHead();
        ct.autoCalculate(IToCalc.SCOPE_CALCULATION, 1);// x + 1 = 2
        ct.autoCalculate(IToCalc.SCOPE_CALCULATION, 1);// x + 1 + -1 * 2 = 0
        //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        //append "x = 1" which needs 6 steps (+ one level down) to justify
        CalcFormula in = new CalcFormula();
        in.setText("x = 1");
        ct.appendFormula(in);//<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
        cce = ((MockIToUser) user).getCalcChangedEvent();
        assertEquals("unchanged", cce.getLastUnchangedFormula().toSMLString(),
                "([1],Res)");
        assertEquals("deleted", cce.getLastDeletedFormula().toSMLString(),
                "([1],Res)");
        assertEquals("generated", cce.getLastGeneratedFormula().toSMLString(),
                "([3,2],Res)");

        //the inserted formulae _MUST_ be displayed (but not on level 3 !):
        Vector elems = cce.getLastUnchangedFormula().getFormulaeFromTo(
                cce.getLastGeneratedFormula(), new Integer(3), false);
        ICalcElement fch = null;
        for (int i = 0; i < elems.size(); i++) {
            fch = (CalcFormula) elems.elementAt(i);
                System.out.println(".step " + i + ": "
                        + ((CalcFormula) fch).getPosition().toSMLString()
                        + "  " + fch.toSMLString());
        }
        assertEquals("CalcElements to be shown:", elems.size(), 8);
        logger_.debug("---END isac.bridge.TestBridgeInput#testAppendOther2");
    }

    /**
     * uses the example 'IsacCore/Tests/minipbl_with_mini-subpbl' comprises the
     * usecase \label{SOLVE:MANUAL:FORMULA:enter} see
     * smltest/FE-interface/interface.sml "...enter} NOTok"
     */
    public void testAppendNotOk() throws Exception {
        System.out
                .println("---BEGIN isac.bridge.TestBridgeInput#testAppendNotOk");

        //load the file of the example,
        //see TestKEStoreServices#testLoadContentMinisubpbl
        KEStoreServices ks = new KEStoreServices("isac/xmldata");
        Vector vec = ks.loadContent("exp", "exp_IsacCore_Tests_1b.xml");
        //parse out the Formalization
        //see TestFormalizationDigest#testParseExplMini
        String xmlstr = (String) vec.firstElement();
        FormalizationDigest fd = new FormalizationDigest();
        Formalization fmz = fd.parseFormalization((String) xmlstr);
        System.out.println(".end step_a-----");

        //start the calculation
        MathEngine.init("localhost");
        MathEngine me = MathEngine.getMathEngine();

        IToCalc ct = me.getCalcTree(fmz);
        IToUser user = new MockIToUser();
        ct.addDataChangeListener(user);
        CalcChanged cce = null;
        System.out.println(".end step_a-----");
        //end step_a
        // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        //do the first two steps
        ct.completeCalcHead();
        ct.autoCalculate(IToCalc.SCOPE_CALCULATION, 1);// x + 1 = 2
        ct.autoCalculate(IToCalc.SCOPE_CALCULATION, 1);// x + 1 + -1 * 2 = 0
        //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        //append "x = 4711" which cannot be derived
        CalcFormula in = new CalcFormula();
        in.setText("x = 4711");
        ct.appendFormula(in);//<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
        //zum Zurueckmelden von <ERROR> no derivation found </ERROR>
        //TODO brauchen wir schon den neuen CalcChanged !!!

        logger_.debug("---END isac.bridge.TestBridgeInput#testAppendNotOk");
    }

    /**
     * uses the example 'IsacCore/Tests/minipbl_with_mini-subpbl' comprises the
     * usecase \label{SOLVE:MANUAL:FORMULA:enter} see
     * smltest/FE-interface/interface.sml "...replace} right"
     */
    public void testReplaceRight() throws Exception {
        System.out
                .println("---BEGIN isac.bridge.TestBridgeInput#testReplaceRight");

        //load the file of the example,
        //see TestKEStoreServices#testLoadContentMinisubpbl
        KEStoreServices ks = new KEStoreServices("isac/xmldata");
        Vector vec = ks.loadContent("exp", "exp_IsacCore_Tests_1b.xml");
        //parse out the Formalization
        //see TestFormalizationDigest#testParseExplMini
        String xmlstr = (String) vec.firstElement();
        FormalizationDigest fd = new FormalizationDigest();
        Formalization fmz = fd.parseFormalization((String) xmlstr);
        System.out.println(".end step_a-----");

        //start the calculation
        MathEngine.init("localhost");
        MathEngine me = MathEngine.getMathEngine();

        IToCalc ct = me.getCalcTree(fmz);
        IToUser user = new MockIToUser();
        ct.addDataChangeListener(user);
        CalcChanged cce = null;
        System.out.println(".end step_a-----");
        //end step_a
        // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        ct.autoCalculate(IToCalc.SCOPE_CALCULATION, 0);
        cce = ((MockIToUser) user).getCalcChangedEvent();
        // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        //replace "-1 + x = 0" by "-1 + x = 0", ie. no changes
        Position pos = new Position();//([2],Res)
        pos.addInt("2");
        pos.setKind("Res");
        ICalcIterator ci = new CalcIterator((CalcTree) ct, pos);
        ct.moveActiveFormula(ci);

        CalcFormula in = new CalcFormula();
        in.setText("-1 + x = 0");
        // WN050208 works only, if startet as only test in this JUnit (other
        // outcommented)
        // WN050408 ^^^^^^^^^^^^^^^^^^^^^^????????????????cannot reproduce!!!
        ct.replaceFormula(in);//<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
        // <ERROR> formula not changed </ERROR> thus ct _NOT_ changed !

        Vector elems = cce.getLastUnchangedFormula().getFormulaeFromTo(
                cce.getLastGeneratedFormula(), new Integer(3), false);
        assertEquals("calculation not affected; still elems.size():", elems
                .size(), 10);

        logger_.debug("---END isac.bridge.TestBridgeInput#testReplaceRight");
    }

    /**
     * uses the example 'IsacCore/Tests/minipbl_with_mini-subpbl' comprises the
     * usecase \label{SOLVE:MANUAL:FORMULA:enter} see
     * smltest/FE-interface/interface.sml "...replace} other"
     */
    public void testReplaceOther1() throws Exception {
        System.out
                .println("---BEGIN isac.bridge.TestBridgeInput#testReplaceOther1");

        //load the file of the example,
        //see TestKEStoreServices#testLoadContentMinisubpbl
        KEStoreServices ks = new KEStoreServices("isac/xmldata");
        Vector vec = ks.loadContent("exp", "exp_IsacCore_Tests_1b.xml");
        //parse out the Formalization
        //see TestFormalizationDigest#testParseExplMini
        String xmlstr = (String) vec.firstElement();
        FormalizationDigest fd = new FormalizationDigest();
        Formalization fmz = fd.parseFormalization((String) xmlstr);
        System.out.println(".end step_a-----");

        //start the calculation
        MathEngine.init("localhost");
        MathEngine me = MathEngine.getMathEngine();

        IToCalc ct = me.getCalcTree(fmz);
        IToUser user = new MockIToUser();
        ct.addDataChangeListener(user);
        CalcChanged cce = null;
        System.out.println(".end step_a-----");
        //end step_a
        // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        //autoCalculate
        ct.autoCalculate(IToCalc.SCOPE_CALCULATION, 0);
        cce = ((MockIToUser) user).getCalcChangedEvent();
        // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        //replace "x - 1 = 0" which needs 11 steps (on same level) to justify
        Position pos = new Position();//([2],Res)
        pos.addInt("2");
        pos.setKind("Res");
        ICalcIterator ci = new CalcIterator((CalcTree) ct, pos);
        ct.moveActiveFormula(ci);

        CalcFormula in = new CalcFormula();
        in.setText("x - 1 = 0");
        ct.replaceFormula(in);//<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
        cce = ((MockIToUser) user).getCalcChangedEvent();
        assertEquals("unchanged", cce.getLastUnchangedFormula().toSMLString(),
                "([1],Res)");
        assertEquals("deleted", cce.getLastDeletedFormula().toSMLString(),
                "([],Res)");
        assertEquals("generated", cce.getLastGeneratedFormula().toSMLString(),
                "([2],Res)");

        //the inserted formulae need _NOT_ to be displayed:
        Vector elems = cce.getLastUnchangedFormula().getFormulaeFromTo(
                cce.getLastGeneratedFormula(), new Integer(0), //not displayed
                false);
        assertEquals("the input formula is returned:", ((CalcFormula) elems
                .firstElement()).toSMLString(), "x - 1 = 0");
        assertEquals("only _one_ formula returned", elems.size(), 1);

        //but there are 11 steps in between
        elems = cce.getLastUnchangedFormula().getFormulaeFromTo(
                cce.getLastGeneratedFormula(), new Integer(2), // displayed
                false);
        assertEquals("elems could be displayed ", elems.size(), 11);
        //get_interval returns ([], Res) too much ! (see
        // smltest/FE-interface/interface.sml)

        logger_.debug("---END isac.bridge.TestBridgeInput#testReplaceOther1");
    }

    /**
     * uses the example 'IsacCore/Tests/minipbl_with_mini-subpbl' comprises the
     * usecase \label{SOLVE:MANUAL:FORMULA:enter} see
     * smltest/FE-interface/interface.sml"...replace} other 2"
     */
    public void testReplaceOther2() throws Exception {
        System.out
                .println("---BEGIN isac.bridge.TestBridgeInput#testReplaceOther2");

        //load the file of the example,
        //see TestKEStoreServices#testLoadContentMinisubpbl
        KEStoreServices ks = new KEStoreServices("isac/xmldata");
        Vector vec = ks.loadContent("exp", "exp_IsacCore_Tests_1b.xml");
        //parse out the Formalization
        //see TestFormalizationDigest#testParseExplMini
        String xmlstr = (String) vec.firstElement();
        FormalizationDigest fd = new FormalizationDigest();
        Formalization fmz = fd.parseFormalization((String) xmlstr);
        System.out.println(".end step_a-----");

        //start the calculation
        MathEngine.init("localhost");
        MathEngine me = MathEngine.getMathEngine();

        IToCalc ct = me.getCalcTree(fmz);
        IToUser user = new MockIToUser();
        ct.addDataChangeListener(user);
        CalcChanged cce = null;
        System.out.println(".end step_a-----");
        //end step_a
        // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        //autoCalculate
        ct.autoCalculate(IToCalc.SCOPE_CALCULATION, 0);
        cce = ((MockIToUser) user).getCalcChangedEvent();
        // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        //replace "x = 1" which needs 6 steps (+ one level down) to justify
        Position pos = new Position();//([2],Res)
        pos.addInt("2");
        pos.setKind("Res");
        ICalcIterator ci = new CalcIterator((CalcTree) ct, pos);
        ct.moveActiveFormula(ci);

        CalcFormula in = new CalcFormula();
        in.setText("x = 1");
        ct.replaceFormula(in);//<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
        cce = ((MockIToUser) user).getCalcChangedEvent();
        assertEquals("unchanged", cce.getLastUnchangedFormula().toSMLString(),
                "([1],Res)");
        assertEquals("deleted", cce.getLastDeletedFormula().toSMLString(),
                "([],Res)");
        assertEquals("generated", cce.getLastGeneratedFormula().toSMLString(),
                "([3,2],Res)");

        //the inserted formulae _MUST_ be displayed (but not on level 3 !):
        Vector elems = cce.getLastUnchangedFormula().getFormulaeFromTo(
                cce.getLastGeneratedFormula(), new Integer(3), false);
        CalcFormula fch = null;
        for (int i = 0; i < elems.size(); i++) {
            fch = (CalcFormula) elems.elementAt(i);
                System.out.println(".step " + i + ": "
                        + fch.getPosition().toSMLString()
                        + "  " + fch.toSMLString());
        }
        assertEquals("CalcElements to be shown:", elems.size(), 8);

        logger_.debug("---END isac.bridge.TestBridgeInput#testReplaceOther2");
    }

    /**
     * uses the example 'IsacCore/Tests/minipbl_with_mini-subpbl' comprises the
     * usecase \label{SOLVE:MANUAL:FORMULA:enter} see
     * smltest/FE-interface/interface.sml "...replace} NOTok"
     */
    public void testReplaceNotOk() throws Exception {
        System.out
                .println("---BEGIN isac.bridge.TestBridgeInput#testReplaceNotOk");

        //load the file of the example,
        //see TestKEStoreServices#testLoadContentMinisubpbl
        KEStoreServices ks = new KEStoreServices("isac/xmldata");
        Vector vec = ks.loadContent("exp", "exp_IsacCore_Tests_1b.xml");
        //parse out the Formalization
        //see TestFormalizationDigest#testParseExplMini
        String xmlstr = (String) vec.firstElement();
        FormalizationDigest fd = new FormalizationDigest();
        Formalization fmz = fd.parseFormalization((String) xmlstr);
        System.out.println(".end step_a-----");

        //start the calculation
        MathEngine.init("localhost");
        MathEngine me = MathEngine.getMathEngine();

        IToCalc ct = me.getCalcTree(fmz);
        IToUser user = new MockIToUser();
        ct.addDataChangeListener(user);
        CalcChanged cce = null;
        System.out.println(".end step_a-----");
        //end step_a
        // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        //autoCalculate
        ct.autoCalculate(IToCalc.SCOPE_CALCULATION, 0);
        cce = ((MockIToUser) user).getCalcChangedEvent();
        // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        //replace "x = 4711" which cannot be derived
        Position pos = new Position();//([2],Res)
        pos.addInt("2");
        pos.setKind("Res");
        ICalcIterator ci = new CalcIterator((CalcTree) ct, pos);
        ct.moveActiveFormula(ci);

        CalcFormula in = new CalcFormula();
        in.setText("x = 4711");
        ct.replaceFormula(in);//<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
        //zum Zurueckmelden von <ERROR> no derivation found </ERROR>
        //TODO brauchen wir schon den neuen CalcChanged !!!

        logger_.debug("---END isac.bridge.TestBridgeInput#testReplaceNotOk");
    }

}