package isac.util;

 import java.io.IOException;
 import java.net.Socket;
 import java.net.ServerSocket;
import java.rmi.server.RMISocketFactory;

 /**
 * @author  Ralf Kellermann
 * @date 2003-08-26
 * @time 18:17
 */
 
/**
 * for new RMI between WindowApplication and ObjectManager
 * during work on problems 0702 with RMI at htl
 * @see isac.session.WatchdogThread
 * @see isac.util.WindowApplicationRMISocketFactory
 */
 public class FixedPortRMISocketFactory extends RMISocketFactory
 {
	 	private int port_;
	 	
	 	public FixedPortRMISocketFactory(int port_to_use)
	 	{
	 		super();
	 		port_ = port_to_use;
	 	}
	 	
         /**
         * Creates a client socket connected to the specified host and port
         * @param host  the host name
         * @param port  the port number
         * @return a socket connected to the specified host and port.
         * @exception IOException if an I/O error occurs during socket creation
         */
         public Socket createSocket(String host, int port)
                 throws IOException
                 
                 
         {
                 //RK for RMI htl: System.out.println("creating socket to host : " 
                 //                + host + " on port " + port);
                 return new Socket(host, port);

         }
 
         /**
         * Create a server socket on the specified port (port 0 indicates
         * an anonymous port) and writes out some debugging info
         * @param port  the port number
         * @return  the server socket on the specified port
         * @exception IOException  if an I/O error occurs during server socket
         * creation
         */
         public ServerSocket createServerSocket(int port)
                 throws IOException
         {
                 port = port_;
                 //RK for RMI at htl: 
                 //System.out.println("creating ServerSocket on port " + port);
                 return new ServerSocket(port);
         }
}

